from .book_authentication_descriptor import BookAuthenticationDescriptor
from .book_custom_authentication_descriptor import \
    BookCustomAuthenticationDescriptor
from .book_oauth_authentication_descriptor import (
    BookOAuthAuthenticationDescriptor, OAuthFlow, OAuthProvider,
    OAuthTokenFunction)
from .credential_descriptor import CredentialDescriptor
from .oauth_argument_descriptor import OauthArgumentDescriptor
